PROVIDER = "bedrock"

MODEL_TITAN_EXPRESS = "amazon.titan-text-express-v1"
EMBEDDING_MODEL_TITAN = "amazon.titan-embed-text-v1"
EMBEDDING_MODEL_TITAN_V2 = "amazon.titan-embed-text-v2:0"

MODEL_ANTHROPIC_SONNET = "anthropic.claude-3-sonnet-20240229-v1:0"
MODEL_ANTHROPIC_HAIKU = "anthropic.claude-3-haiku-20240307-v1:0"
EMBEDDING_MODEL_COHERE_ENG = "cohere.embed-english-v3"
EMBEDDING_MODEL_COHERE_MULTI = "cohere.embed-multilingual-v3"

MODEL_LLAMA_70B = "meta.llama3-70b-instruct-v1:0"

DEFAULT_MODEL = MODEL_TITAN_EXPRESS
DEFAULT_EMBEDDING_MODEL = EMBEDDING_MODEL_TITAN
